/************************************************************************/
/*				                        		*/
/*	RDKcK_Line_Demo                 		                */
/*						                  	*/
/*					                        	*/
/************************************************************************/
/*	Author: Jordan Retz						*/
/*	Copyright 2012, Digilent Inc.					*/
/************************************************************************/
/*  Module Description: 			             		*/
/*					                        	*/
/* This module implements a simple demo application that demonstrates   */
/* how to program the RDKcK Robot as a Line Following robot.            */
/*									*/
/*	Functionality:							*/
/*									*/
/* The application uses the PmodLS1, four IR proximity sensors, and     */
/* servo motors. Based on the outputs of the four sensors attached to   */
/* the Pmod, the robot will alter it's direction to stay on a line.     */
/* The robot will work best if the line is black electric tape on a     */
/* white surface. This demo assumes that an MX3cK is being used.        */
/*                                                                      */
/*                                                                      */
/*      Instruction     FRS MRS MLS FLS    Function                     */
/*          0            0   0   0   0     Stop                         */
/*          1            0   0   0   1     Turn Right                   */
/*          2            0   0   1   0     Continue                     */
/*          3            0   0   1   1     Turn Right                   */
/*          4            0   1   0   0     Continue                     */
/*          5            0   1   0   1     Continue                     */
/*          6            0   1   1   0     Continue                     */
/*          7            0   1   1   1     Turn Right                   */
/*          8            1   0   0   0     Turn Left                    */
/*          9            1   0   0   1     Continue                     */
/*          10           1   0   1   0     Continue                     */
/*          11           1   0   1   1     Turn Right                   */
/*          12           1   1   0   0     Turn Left                    */
/*          13           1   1   0   1     Turn Left                    */
/*          14           1   1   1   0     Turn Left                    */
/*          15           1   1   1   1     Move Foward                  */
/************************************************************************/
/*  Revision History:			        			*/
/*					                        	*/
/*	4/12/2012 (JordanR): created		       			*/
/*						                     	*/
/************************************************************************/

/* ------------------------------------------------------------ */
/*                Include File Definitions                      */
/* ------------------------------------------------------------ */
#include <Servo.h>

/* ------------------------------------------------------------ */
/*				Local Type Definitions		*/
/* ------------------------------------------------------------ */
/*Pmod pin connections for MX3cK
*/
//JC bottom
#define servoPinL        22                  //left servo
#define servoPinR        23                  //right servo

//JE top
/* The sensor pins are referred to as farRight, midRight, midLeft, and farLeft.
* These should be connected to S1, S2, S3, S4, respectively, on the PmodLS1.
* Left and right are determined looking down on robot with sensor  bracket forward.
*/
#define farRight        32                  //sensor1
#define midRight        33                  //sensor2
#define midLeft         34                  //sensor3
#define farLeft         35                  //sensor4

/* ------------------------------------------------------------ */
/*                Global Variables                              */
/* ------------------------------------------------------------ */
/*Define Servos*/
Servo      left;
Servo      right;
int        instruction;
int        prevInstruction;

/* ------------------------------------------------------------ */
/*                Forward Declarations                          */
/* ------------------------------------------------------------ */

/* ------------------------------------------------------------ */
/*                Procedure Definitions                         */
/* ------------------------------------------------------------ */
/***  void setup()
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Initialize the system.
*/
void setup()
{
  /*Setup Input/Output on Pins*/
  pinMode(servoPinL, OUTPUT);
  pinMode(servoPinR, OUTPUT);
  
  pinMode(farRight, INPUT); 
  pinMode(midRight, INPUT); 
  pinMode(midLeft, INPUT); 
  pinMode(farLeft, INPUT); 
  
  forward();
  prevInstruction = 15;
}

/* ------------------------------------------------------------ */
/***  void loop();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Main application loop.
*/
void loop()
{
  instruction = checkSensors();
  
  
  if(instruction != prevInstruction)
  {
    if(instruction == 0)
    {
      stopRobot();
      delay(20);
    }
    else if(instruction == 3 || instruction == 7 || instruction == 11 || instruction == 1 || instruction == 5)
    {
      turnRight();
      delay(20);
    }
    else if(instruction == 12 || instruction == 13 || instruction == 14 || instruction == 8 || instruction == 10)
    {
      turnLeft();
      delay(20);
    }
    else
    {
      forward();
      delay(20);
    }
    
    prevInstruction = instruction;
  }
}


/* ------------------------------------------------------------ */
/***  int checkSensors();
**
**  Parameters:
**    none
**
**  Return Value:
**  none
**
**  Errors:
**  none
**
**  Description:
**    Pull the states off each pin and return int with configuration.
*/
int checkSensors()
{
    int sensor = 0;
    byte sensorConfig = B00000000;

    byte frs = digitalRead(farRight)*(0x08);
    byte mrs = digitalRead(midRight)*(0x04);
    byte mls = digitalRead(midLeft)*(0x02);
    byte fls = digitalRead(farLeft)*(0x01);
    
    sensorConfig = frs | mrs | mls | fls;
    
    return int(sensorConfig);
}

/* ------------------------------------------------------------ */
/***	void stopRobot();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Stops robot by finding Servo position and reducing or increasing to 90. 
**    Then detaches pins.
*/
void stopRobot()
{  
  left.write(90);
  right.write(90);
  
  left.detach();
  right.detach(); 
}

/* ------------------------------------------------------------ */
/***	void forward();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Moves robot forward
*/
void forward()
{
  left.attach(servoPinL);
  right.attach(servoPinR);
  
  left.write(107);
  right.write(73);
}

/* ------------------------------------------------------------ */
/***	void turnRight();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**    Turns robot right
*/
void turnRight()
{
  left.attach(servoPinL);
  right.attach(servoPinR);
  
  left.write(107);
  right.write(107);
}

/* ------------------------------------------------------------ */
/***	void turnLeft();
**
**  Parameters:
**    none
**
**  Return Value:
**    none
**
**  Errors:
**    none
**
**  Description:
**   Turns robot left
*/
void turnLeft()
{
  left.attach(servoPinL);
  right.attach(servoPinR);
  
  left.write(73);
  right.write(73);  
}
